package net.wanji.opt.servicev2.implv2;

import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.BaseEnum;
import net.wanji.opt.dao.mapper.CrossMapper;
import net.wanji.opt.dao.mapper.GreenwaveHistoryMapper;
import net.wanji.opt.po.base.CrossLaneDataHistPoExtend;
import net.wanji.opt.servicev2.CrossService;
import net.wanji.opt.synthesis.pojo.*;
import net.wanji.opt.vo2.CrossBaseInfoVO;
import net.wanji.opt.vo2.CrossStatusDistributionVO;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

/**
 * <p>
 * 监测详情-路口事件详情
 *
 * @author zhengyifan
 * @date 2025/3/15
 */
@Service
@Slf4j
public class CrossServiceImpl implements CrossService {
    @Resource
    private CrossMapper crossMapper;
    @Resource
    private GreenwaveHistoryMapper greenwaveHistoryMapper;

    @Override
    public Map<String, Object> getCrossStatusDistribution(String crossID, String date, String groupType, Integer objectType, String condition) {
        Map<String, Object> resultMap = new HashMap<>();

        if (objectType == 1) {
            // 方向级
            // condition是方向--1 3 5 7
            String[] conditions = condition.split(";");
            for (String c : conditions) {
                List<CrossStatusDistributionVO> list = getCrossStatusDistribution_SingleCondition(crossID, date, groupType, objectType, c);
                resultMap.put(c + "List", list);
            }
        } else if (objectType == 2) {
            // 转向级
            // condition是转向--dir:l dir:r dir:n dir:u
            String[] conditions = condition.split(";");
            for (String c : conditions) {
                List<CrossStatusDistributionVO> list = getCrossStatusDistribution_SingleCondition(crossID, date, groupType, objectType, c);
                String[] cs = c.split(":");
                String namePart1 = BaseEnum.SignalDirectionEnum.getNameByCode(Integer.parseInt(cs[0]));
                String namePart2 = BaseEnum.TurnTypeEnum.getNameByCode(Integer.parseInt(cs[1]));
                String name = namePart1 + namePart2;
                resultMap.put(name, list);
            }
        } else if (objectType == 3) {
            // 车道级
            // condition是dir:turn:laneNum
            String[] conditions = condition.split(";");
            for (String c : conditions) {
                List<CrossStatusDistributionVO> list = getCrossStatusDistribution_SingleCondition(crossID, date, groupType, objectType, c);
                String[] cs = c.split(":");
                String namePart1 = BaseEnum.SignalDirectionEnum.getNameByCode(Integer.parseInt(cs[0]));
                String namePart2 = BaseEnum.TurnTypeEnum.getNameByCode(Integer.parseInt(cs[1]));
                String namePart3 = cs[2];
                String name = namePart1 + namePart2 + namePart3;
                resultMap.put(name, list);
            }
        } else if (objectType == 4) {
            // 路口级
            // 无condition
            List<CrossStatusDistributionVO> list = getCrossStatusDistribution_SingleCondition(crossID, date, groupType, objectType, "");
            resultMap.put("cross_list", list);
        }
        return resultMap;
    }

    /**
     * 路口状态分布_单条件
     *
     * @param crossID       路口ID
     * @param date          日期
     * @param groupType     时间粒度 参数要求：0--5分钟 1--15分钟 2--30分钟 3--60分钟
     * @param objectType    范围 参数要求：1--方向级 2--转向级 3--车道级 4--路口级
     * @param condition     条件
     */
    @Override
    public List<CrossStatusDistributionVO> getCrossStatusDistribution_SingleCondition(String crossID, String date, String groupType, Integer objectType, String condition) {
        Map<String, Object> params = new HashMap<>();
        params.put("crossId", crossID);
        params.put("startDate", date + " 00:00:00");
        params.put("endDate", date + " 23:59:59");
        params.put("groupType", groupType);
        params.put("objectType", objectType);
        // 方向级时多传一个dir参数
        if (objectType == 1) params.put("dir", condition);

        List<CrossLaneDataHistPoExtend> firstList = greenwaveHistoryMapper.findCrossObjectIndex(params);
        List<CrossStatusDistributionVO> results = new ArrayList<>();

        for (CrossLaneDataHistPoExtend c: firstList) {
            CrossStatusDistributionVO cd = new CrossStatusDistributionVO();

            cd.setSpeed(c.getSpeed());
            cd.setStartTime(c.getStartTime().toString().substring(11, 16));
            cd.setDelayTime(c.getDelayTime());
            cd.setTurnType(c.getTurnType());
            cd.setTurnTypeName(BaseEnum.TurnTypeEnum.getNameByCode(c.getTurnType()));
            cd.setDirType(c.getDirType());
            cd.setDirTypeName(BaseEnum.SignalDirectionEnum.getNameByCode(c.getDirType()));
            cd.setStopTimes(c.getStopTimes());
            cd.setQueueLength(c.getAvgQueueLength());
            cd.setLaneNo(c.getLaneNo());

            results.add(cd);
        }

        // 根据筛选条件筛选一次结果
        if (objectType == 2) {
            // 转向级
            String[] con = condition.split(":");
            results.removeIf(c -> c.getDirType() != Integer.parseInt(con[0]));
            results.removeIf(c -> !Objects.equals(c.getTurnType(), BaseEnum.TurnTypeEnum.getIntCodeByStrCode(con[1])));

        } else if (objectType == 3) {
            // 车道级
            String[] con = condition.split(":");
            results.removeIf(c -> c.getDirType() != Integer.parseInt(con[0]));
            results.removeIf(c -> c.getTurnType() != Integer.parseInt(con[1]));
            results.removeIf(c -> c.getLaneNo() != Integer.parseInt(con[2]));
        }
        return results;
    }

    /**
     * 路口实时告警
     *
     * @param crossID 路口ID
     * @param time    可选参数
     * @return
     */
    @Override
    public List<CrossRealTimeAlarmEntity> getCrossRealTimeAlarm(String crossID, String time) {
        return crossMapper.getCrossRealTimeAlarms(crossID, time);
    }

    @Override
    public List<CrossStatusDisOptTimeEntity> getOptTimeList(String crossID) {
        return crossMapper.getOptTimes(crossID);
    }

    /**
     * 路口基础信息————方向、转向、车道信息
     *
     * @param crossID 路口ID
     */
    @Override
    public CrossBaseInfoVO getCrossBaseInfos(String crossID) {
        List<CrossBaseInfoVO> crossBaseInfoVOS = new ArrayList<>();

        List<CrossDirectionInfoEntity> directionInfoEntities = crossMapper.getCrossDirectionInfo(crossID);
        List<CrossTurnInfoEntity> turnInfoEntities = crossMapper.getCrossTurnInfo(crossID);
        List<CrossLaneInfoEntity> laneInfoEntities = crossMapper.getCrossLaneInfo(crossID);

        CrossBaseInfoVO crossBaseInfoVO = new CrossBaseInfoVO();
        crossBaseInfoVO.setCrossID(crossID);
        crossBaseInfoVO.setDirectionInfo(directionInfoEntities);
        crossBaseInfoVO.setTurnInfo(turnInfoEntities);
        crossBaseInfoVO.setLaneInfo(laneInfoEntities);

        return crossBaseInfoVO;
    }

}
